# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
import base64
from typing import TYPE_CHECKING, Any, Mapping
from datetime import datetime
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Headers,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._models import FinalRequestOptions
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import M3terError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import (
        bills,
        plans,
        usage,
        users,
        events,
        meters,
        charges,
        accounts,
        balances,
        counters,
        pricings,
        products,
        webhooks,
        bill_jobs,
        contracts,
        currencies,
        statements,
        bill_config,
        commitments,
        plan_groups,
        aggregations,
        data_exports,
        account_plans,
        custom_fields,
        debit_reasons,
        lookup_tables,
        authentication,
        credit_reasons,
        plan_templates,
        resource_groups,
        counter_pricings,
        plan_group_links,
        external_mappings,
        transaction_types,
        counter_adjustments,
        organization_config,
        permission_policies,
        compound_aggregations,
        integration_configurations,
        notification_configurations,
        scheduled_event_configurations,
    )
    from .resources.plans import PlansResource, AsyncPlansResource
    from .resources.events import EventsResource, AsyncEventsResource
    from .resources.meters import MetersResource, AsyncMetersResource
    from .resources.charges import ChargesResource, AsyncChargesResource
    from .resources.accounts import AccountsResource, AsyncAccountsResource
    from .resources.counters import CountersResource, AsyncCountersResource
    from .resources.pricings import PricingsResource, AsyncPricingsResource
    from .resources.products import ProductsResource, AsyncProductsResource
    from .resources.webhooks import WebhooksResource, AsyncWebhooksResource
    from .resources.bill_jobs import BillJobsResource, AsyncBillJobsResource
    from .resources.contracts import ContractsResource, AsyncContractsResource
    from .resources.currencies import CurrenciesResource, AsyncCurrenciesResource
    from .resources.bill_config import BillConfigResource, AsyncBillConfigResource
    from .resources.bills.bills import BillsResource, AsyncBillsResource
    from .resources.commitments import CommitmentsResource, AsyncCommitmentsResource
    from .resources.plan_groups import PlanGroupsResource, AsyncPlanGroupsResource
    from .resources.usage.usage import UsageResource, AsyncUsageResource
    from .resources.users.users import UsersResource, AsyncUsersResource
    from .resources.aggregations import AggregationsResource, AsyncAggregationsResource
    from .resources.account_plans import AccountPlansResource, AsyncAccountPlansResource
    from .resources.custom_fields import CustomFieldsResource, AsyncCustomFieldsResource
    from .resources.debit_reasons import DebitReasonsResource, AsyncDebitReasonsResource
    from .resources.authentication import AuthenticationResource, AsyncAuthenticationResource
    from .resources.credit_reasons import CreditReasonsResource, AsyncCreditReasonsResource
    from .resources.plan_templates import PlanTemplatesResource, AsyncPlanTemplatesResource
    from .resources.resource_groups import ResourceGroupsResource, AsyncResourceGroupsResource
    from .resources.counter_pricings import CounterPricingsResource, AsyncCounterPricingsResource
    from .resources.plan_group_links import PlanGroupLinksResource, AsyncPlanGroupLinksResource
    from .resources.balances.balances import BalancesResource, AsyncBalancesResource
    from .resources.external_mappings import ExternalMappingsResource, AsyncExternalMappingsResource
    from .resources.transaction_types import TransactionTypesResource, AsyncTransactionTypesResource
    from .resources.counter_adjustments import CounterAdjustmentsResource, AsyncCounterAdjustmentsResource
    from .resources.organization_config import OrganizationConfigResource, AsyncOrganizationConfigResource
    from .resources.permission_policies import PermissionPoliciesResource, AsyncPermissionPoliciesResource
    from .resources.compound_aggregations import CompoundAggregationsResource, AsyncCompoundAggregationsResource
    from .resources.statements.statements import StatementsResource, AsyncStatementsResource
    from .resources.data_exports.data_exports import DataExportsResource, AsyncDataExportsResource
    from .resources.integration_configurations import (
        IntegrationConfigurationsResource,
        AsyncIntegrationConfigurationsResource,
    )
    from .resources.lookup_tables.lookup_tables import LookupTablesResource, AsyncLookupTablesResource
    from .resources.notification_configurations import (
        NotificationConfigurationsResource,
        AsyncNotificationConfigurationsResource,
    )
    from .resources.scheduled_event_configurations import (
        ScheduledEventConfigurationsResource,
        AsyncScheduledEventConfigurationsResource,
    )

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "M3ter", "AsyncM3ter", "Client", "AsyncClient"]

from .types import AuthenticationGetBearerTokenResponse


class M3ter(SyncAPIClient):
    # client options
    api_key: str
    api_secret: str
    token: str | None
    token_expiry: datetime | None
    org_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        token_expiry: datetime | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous M3ter client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `M3TER_API_KEY`
        - `api_secret` from `M3TER_API_SECRET`
        - `token` from `M3TER_API_TOKEN`
        - `org_id` from `M3TER_ORG_ID`
        """
        if api_key is None:
            api_key = os.environ.get("M3TER_API_KEY")
        if api_key is None:
            raise M3terError(
                "The api_key client option must be set either by passing api_key to the client or by setting the M3TER_API_KEY environment variable"
            )
        self.api_key = api_key

        if api_secret is None:
            api_secret = os.environ.get("M3TER_API_SECRET")
        if api_secret is None:
            raise M3terError(
                "The api_secret client option must be set either by passing api_secret to the client or by setting the M3TER_API_SECRET environment variable"
            )
        self.api_secret = api_secret

        if token is None:
            token = os.environ.get("M3TER_API_TOKEN")
        self.token = token
        self.token_expiry = token_expiry

        if org_id is None:
            org_id = os.environ.get("M3TER_ORG_ID")
        if org_id is None:
            raise M3terError(
                "The org_id client option must be set either by passing org_id to the client or by setting the M3TER_ORG_ID environment variable"
            )
        self.org_id = org_id

        if base_url is None:
            base_url = os.environ.get("M3TER_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.m3ter.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def authentication(self) -> AuthenticationResource:
        from .resources.authentication import AuthenticationResource

        return AuthenticationResource(self)

    @cached_property
    def accounts(self) -> AccountsResource:
        from .resources.accounts import AccountsResource

        return AccountsResource(self)

    @cached_property
    def account_plans(self) -> AccountPlansResource:
        from .resources.account_plans import AccountPlansResource

        return AccountPlansResource(self)

    @cached_property
    def aggregations(self) -> AggregationsResource:
        from .resources.aggregations import AggregationsResource

        return AggregationsResource(self)

    @cached_property
    def balances(self) -> BalancesResource:
        from .resources.balances import BalancesResource

        return BalancesResource(self)

    @cached_property
    def bills(self) -> BillsResource:
        from .resources.bills import BillsResource

        return BillsResource(self)

    @cached_property
    def bill_config(self) -> BillConfigResource:
        from .resources.bill_config import BillConfigResource

        return BillConfigResource(self)

    @cached_property
    def commitments(self) -> CommitmentsResource:
        from .resources.commitments import CommitmentsResource

        return CommitmentsResource(self)

    @cached_property
    def bill_jobs(self) -> BillJobsResource:
        from .resources.bill_jobs import BillJobsResource

        return BillJobsResource(self)

    @cached_property
    def charges(self) -> ChargesResource:
        from .resources.charges import ChargesResource

        return ChargesResource(self)

    @cached_property
    def compound_aggregations(self) -> CompoundAggregationsResource:
        from .resources.compound_aggregations import CompoundAggregationsResource

        return CompoundAggregationsResource(self)

    @cached_property
    def contracts(self) -> ContractsResource:
        from .resources.contracts import ContractsResource

        return ContractsResource(self)

    @cached_property
    def counters(self) -> CountersResource:
        from .resources.counters import CountersResource

        return CountersResource(self)

    @cached_property
    def counter_adjustments(self) -> CounterAdjustmentsResource:
        from .resources.counter_adjustments import CounterAdjustmentsResource

        return CounterAdjustmentsResource(self)

    @cached_property
    def counter_pricings(self) -> CounterPricingsResource:
        from .resources.counter_pricings import CounterPricingsResource

        return CounterPricingsResource(self)

    @cached_property
    def credit_reasons(self) -> CreditReasonsResource:
        from .resources.credit_reasons import CreditReasonsResource

        return CreditReasonsResource(self)

    @cached_property
    def currencies(self) -> CurrenciesResource:
        from .resources.currencies import CurrenciesResource

        return CurrenciesResource(self)

    @cached_property
    def custom_fields(self) -> CustomFieldsResource:
        from .resources.custom_fields import CustomFieldsResource

        return CustomFieldsResource(self)

    @cached_property
    def data_exports(self) -> DataExportsResource:
        from .resources.data_exports import DataExportsResource

        return DataExportsResource(self)

    @cached_property
    def debit_reasons(self) -> DebitReasonsResource:
        from .resources.debit_reasons import DebitReasonsResource

        return DebitReasonsResource(self)

    @cached_property
    def events(self) -> EventsResource:
        from .resources.events import EventsResource

        return EventsResource(self)

    @cached_property
    def external_mappings(self) -> ExternalMappingsResource:
        from .resources.external_mappings import ExternalMappingsResource

        return ExternalMappingsResource(self)

    @cached_property
    def integration_configurations(self) -> IntegrationConfigurationsResource:
        from .resources.integration_configurations import IntegrationConfigurationsResource

        return IntegrationConfigurationsResource(self)

    @cached_property
    def lookup_tables(self) -> LookupTablesResource:
        from .resources.lookup_tables import LookupTablesResource

        return LookupTablesResource(self)

    @cached_property
    def meters(self) -> MetersResource:
        from .resources.meters import MetersResource

        return MetersResource(self)

    @cached_property
    def notification_configurations(self) -> NotificationConfigurationsResource:
        from .resources.notification_configurations import NotificationConfigurationsResource

        return NotificationConfigurationsResource(self)

    @cached_property
    def organization_config(self) -> OrganizationConfigResource:
        from .resources.organization_config import OrganizationConfigResource

        return OrganizationConfigResource(self)

    @cached_property
    def permission_policies(self) -> PermissionPoliciesResource:
        from .resources.permission_policies import PermissionPoliciesResource

        return PermissionPoliciesResource(self)

    @cached_property
    def plans(self) -> PlansResource:
        from .resources.plans import PlansResource

        return PlansResource(self)

    @cached_property
    def plan_groups(self) -> PlanGroupsResource:
        from .resources.plan_groups import PlanGroupsResource

        return PlanGroupsResource(self)

    @cached_property
    def plan_group_links(self) -> PlanGroupLinksResource:
        from .resources.plan_group_links import PlanGroupLinksResource

        return PlanGroupLinksResource(self)

    @cached_property
    def plan_templates(self) -> PlanTemplatesResource:
        from .resources.plan_templates import PlanTemplatesResource

        return PlanTemplatesResource(self)

    @cached_property
    def pricings(self) -> PricingsResource:
        from .resources.pricings import PricingsResource

        return PricingsResource(self)

    @cached_property
    def products(self) -> ProductsResource:
        from .resources.products import ProductsResource

        return ProductsResource(self)

    @cached_property
    def resource_groups(self) -> ResourceGroupsResource:
        from .resources.resource_groups import ResourceGroupsResource

        return ResourceGroupsResource(self)

    @cached_property
    def scheduled_event_configurations(self) -> ScheduledEventConfigurationsResource:
        from .resources.scheduled_event_configurations import ScheduledEventConfigurationsResource

        return ScheduledEventConfigurationsResource(self)

    @cached_property
    def statements(self) -> StatementsResource:
        from .resources.statements import StatementsResource

        return StatementsResource(self)

    @cached_property
    def transaction_types(self) -> TransactionTypesResource:
        from .resources.transaction_types import TransactionTypesResource

        return TransactionTypesResource(self)

    @cached_property
    def usage(self) -> UsageResource:
        from .resources.usage import UsageResource

        return UsageResource(self)

    @cached_property
    def users(self) -> UsersResource:
        from .resources.users import UsersResource

        return UsersResource(self)

    @cached_property
    def webhooks(self) -> WebhooksResource:
        from .resources.webhooks import WebhooksResource

        return WebhooksResource(self)

    @cached_property
    def with_raw_response(self) -> M3terWithRawResponse:
        return M3terWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> M3terWithStreamedResponse:
        return M3terWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        token = self.token
        if token is None:
            return {}
        return {"Authorization": f"Bearer {token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if self.token or headers.get("Authorization") or isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the token to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    @override
    def _prepare_options(
        self,
        options: FinalRequestOptions,  # noqa: ARG002
    ) -> FinalRequestOptions:
        token_valid: bool = self.token is not None and (self.token_expiry is None or self.token_expiry > datetime.now())
        if not options.url.endswith("/oauth/token"):
            if not token_valid:
                auth: str = base64.b64encode(f"{self.api_key}:{self.api_secret}".encode("utf8")).decode("utf8")
                token: AuthenticationGetBearerTokenResponse = self.authentication.get_bearer_token(
                    grant_type="client_credentials", extra_headers={"Authorization": f"Basic {auth}"}
                )
                self.token = token.access_token
                # expiry minus 5 minutes from effective refreshing
                self.token_expiry = datetime.fromtimestamp(token.expires_in - 300)
        return options

    def copy(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        token_expiry: datetime | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            api_secret=api_secret or self.api_secret,
            token=token or self.token,
            token_expiry=token_expiry or self.token_expiry,
            org_id=org_id or self.org_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_org_id_path_param(self) -> str:
        return self.org_id

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncM3ter(AsyncAPIClient):
    # client options
    api_key: str
    api_secret: str
    token: str | None
    org_id: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncM3ter client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `M3TER_API_KEY`
        - `api_secret` from `M3TER_API_SECRET`
        - `token` from `M3TER_API_TOKEN`
        - `org_id` from `M3TER_ORG_ID`
        """
        if api_key is None:
            api_key = os.environ.get("M3TER_API_KEY")
        if api_key is None:
            raise M3terError(
                "The api_key client option must be set either by passing api_key to the client or by setting the M3TER_API_KEY environment variable"
            )
        self.api_key = api_key

        if api_secret is None:
            api_secret = os.environ.get("M3TER_API_SECRET")
        if api_secret is None:
            raise M3terError(
                "The api_secret client option must be set either by passing api_secret to the client or by setting the M3TER_API_SECRET environment variable"
            )
        self.api_secret = api_secret

        if token is None:
            token = os.environ.get("M3TER_API_TOKEN")
        self.token = token

        if org_id is None:
            org_id = os.environ.get("M3TER_ORG_ID")
        if org_id is None:
            raise M3terError(
                "The org_id client option must be set either by passing org_id to the client or by setting the M3TER_ORG_ID environment variable"
            )
        self.org_id = org_id

        if base_url is None:
            base_url = os.environ.get("M3TER_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.m3ter.com"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def authentication(self) -> AsyncAuthenticationResource:
        from .resources.authentication import AsyncAuthenticationResource

        return AsyncAuthenticationResource(self)

    @cached_property
    def accounts(self) -> AsyncAccountsResource:
        from .resources.accounts import AsyncAccountsResource

        return AsyncAccountsResource(self)

    @cached_property
    def account_plans(self) -> AsyncAccountPlansResource:
        from .resources.account_plans import AsyncAccountPlansResource

        return AsyncAccountPlansResource(self)

    @cached_property
    def aggregations(self) -> AsyncAggregationsResource:
        from .resources.aggregations import AsyncAggregationsResource

        return AsyncAggregationsResource(self)

    @cached_property
    def balances(self) -> AsyncBalancesResource:
        from .resources.balances import AsyncBalancesResource

        return AsyncBalancesResource(self)

    @cached_property
    def bills(self) -> AsyncBillsResource:
        from .resources.bills import AsyncBillsResource

        return AsyncBillsResource(self)

    @cached_property
    def bill_config(self) -> AsyncBillConfigResource:
        from .resources.bill_config import AsyncBillConfigResource

        return AsyncBillConfigResource(self)

    @cached_property
    def commitments(self) -> AsyncCommitmentsResource:
        from .resources.commitments import AsyncCommitmentsResource

        return AsyncCommitmentsResource(self)

    @cached_property
    def bill_jobs(self) -> AsyncBillJobsResource:
        from .resources.bill_jobs import AsyncBillJobsResource

        return AsyncBillJobsResource(self)

    @cached_property
    def charges(self) -> AsyncChargesResource:
        from .resources.charges import AsyncChargesResource

        return AsyncChargesResource(self)

    @cached_property
    def compound_aggregations(self) -> AsyncCompoundAggregationsResource:
        from .resources.compound_aggregations import AsyncCompoundAggregationsResource

        return AsyncCompoundAggregationsResource(self)

    @cached_property
    def contracts(self) -> AsyncContractsResource:
        from .resources.contracts import AsyncContractsResource

        return AsyncContractsResource(self)

    @cached_property
    def counters(self) -> AsyncCountersResource:
        from .resources.counters import AsyncCountersResource

        return AsyncCountersResource(self)

    @cached_property
    def counter_adjustments(self) -> AsyncCounterAdjustmentsResource:
        from .resources.counter_adjustments import AsyncCounterAdjustmentsResource

        return AsyncCounterAdjustmentsResource(self)

    @cached_property
    def counter_pricings(self) -> AsyncCounterPricingsResource:
        from .resources.counter_pricings import AsyncCounterPricingsResource

        return AsyncCounterPricingsResource(self)

    @cached_property
    def credit_reasons(self) -> AsyncCreditReasonsResource:
        from .resources.credit_reasons import AsyncCreditReasonsResource

        return AsyncCreditReasonsResource(self)

    @cached_property
    def currencies(self) -> AsyncCurrenciesResource:
        from .resources.currencies import AsyncCurrenciesResource

        return AsyncCurrenciesResource(self)

    @cached_property
    def custom_fields(self) -> AsyncCustomFieldsResource:
        from .resources.custom_fields import AsyncCustomFieldsResource

        return AsyncCustomFieldsResource(self)

    @cached_property
    def data_exports(self) -> AsyncDataExportsResource:
        from .resources.data_exports import AsyncDataExportsResource

        return AsyncDataExportsResource(self)

    @cached_property
    def debit_reasons(self) -> AsyncDebitReasonsResource:
        from .resources.debit_reasons import AsyncDebitReasonsResource

        return AsyncDebitReasonsResource(self)

    @cached_property
    def events(self) -> AsyncEventsResource:
        from .resources.events import AsyncEventsResource

        return AsyncEventsResource(self)

    @cached_property
    def external_mappings(self) -> AsyncExternalMappingsResource:
        from .resources.external_mappings import AsyncExternalMappingsResource

        return AsyncExternalMappingsResource(self)

    @cached_property
    def integration_configurations(self) -> AsyncIntegrationConfigurationsResource:
        from .resources.integration_configurations import AsyncIntegrationConfigurationsResource

        return AsyncIntegrationConfigurationsResource(self)

    @cached_property
    def lookup_tables(self) -> AsyncLookupTablesResource:
        from .resources.lookup_tables import AsyncLookupTablesResource

        return AsyncLookupTablesResource(self)

    @cached_property
    def meters(self) -> AsyncMetersResource:
        from .resources.meters import AsyncMetersResource

        return AsyncMetersResource(self)

    @cached_property
    def notification_configurations(self) -> AsyncNotificationConfigurationsResource:
        from .resources.notification_configurations import AsyncNotificationConfigurationsResource

        return AsyncNotificationConfigurationsResource(self)

    @cached_property
    def organization_config(self) -> AsyncOrganizationConfigResource:
        from .resources.organization_config import AsyncOrganizationConfigResource

        return AsyncOrganizationConfigResource(self)

    @cached_property
    def permission_policies(self) -> AsyncPermissionPoliciesResource:
        from .resources.permission_policies import AsyncPermissionPoliciesResource

        return AsyncPermissionPoliciesResource(self)

    @cached_property
    def plans(self) -> AsyncPlansResource:
        from .resources.plans import AsyncPlansResource

        return AsyncPlansResource(self)

    @cached_property
    def plan_groups(self) -> AsyncPlanGroupsResource:
        from .resources.plan_groups import AsyncPlanGroupsResource

        return AsyncPlanGroupsResource(self)

    @cached_property
    def plan_group_links(self) -> AsyncPlanGroupLinksResource:
        from .resources.plan_group_links import AsyncPlanGroupLinksResource

        return AsyncPlanGroupLinksResource(self)

    @cached_property
    def plan_templates(self) -> AsyncPlanTemplatesResource:
        from .resources.plan_templates import AsyncPlanTemplatesResource

        return AsyncPlanTemplatesResource(self)

    @cached_property
    def pricings(self) -> AsyncPricingsResource:
        from .resources.pricings import AsyncPricingsResource

        return AsyncPricingsResource(self)

    @cached_property
    def products(self) -> AsyncProductsResource:
        from .resources.products import AsyncProductsResource

        return AsyncProductsResource(self)

    @cached_property
    def resource_groups(self) -> AsyncResourceGroupsResource:
        from .resources.resource_groups import AsyncResourceGroupsResource

        return AsyncResourceGroupsResource(self)

    @cached_property
    def scheduled_event_configurations(self) -> AsyncScheduledEventConfigurationsResource:
        from .resources.scheduled_event_configurations import AsyncScheduledEventConfigurationsResource

        return AsyncScheduledEventConfigurationsResource(self)

    @cached_property
    def statements(self) -> AsyncStatementsResource:
        from .resources.statements import AsyncStatementsResource

        return AsyncStatementsResource(self)

    @cached_property
    def transaction_types(self) -> AsyncTransactionTypesResource:
        from .resources.transaction_types import AsyncTransactionTypesResource

        return AsyncTransactionTypesResource(self)

    @cached_property
    def usage(self) -> AsyncUsageResource:
        from .resources.usage import AsyncUsageResource

        return AsyncUsageResource(self)

    @cached_property
    def users(self) -> AsyncUsersResource:
        from .resources.users import AsyncUsersResource

        return AsyncUsersResource(self)

    @cached_property
    def webhooks(self) -> AsyncWebhooksResource:
        from .resources.webhooks import AsyncWebhooksResource

        return AsyncWebhooksResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncM3terWithRawResponse:
        return AsyncM3terWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncM3terWithStreamedResponse:
        return AsyncM3terWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        token = self.token
        if token is None:
            return {}
        return {"Authorization": f"Bearer {token}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    @override
    def _validate_headers(self, headers: Headers, custom_headers: Headers) -> None:
        if headers.get("Authorization") or isinstance(custom_headers.get("Authorization"), Omit):
            return

        raise TypeError(
            '"Could not resolve authentication method. Expected the token to be set. Or for the `Authorization` headers to be explicitly omitted"'
        )

    def copy(
        self,
        *,
        api_key: str | None = None,
        api_secret: str | None = None,
        token: str | None = None,
        org_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            api_secret=api_secret or self.api_secret,
            token=token or self.token,
            org_id=org_id or self.org_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_org_id_path_param(self) -> str:
        return self.org_id

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class M3terWithRawResponse:
    _client: M3ter

    def __init__(self, client: M3ter) -> None:
        self._client = client

    @cached_property
    def authentication(self) -> authentication.AuthenticationResourceWithRawResponse:
        from .resources.authentication import AuthenticationResourceWithRawResponse

        return AuthenticationResourceWithRawResponse(self._client.authentication)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithRawResponse:
        from .resources.accounts import AccountsResourceWithRawResponse

        return AccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def account_plans(self) -> account_plans.AccountPlansResourceWithRawResponse:
        from .resources.account_plans import AccountPlansResourceWithRawResponse

        return AccountPlansResourceWithRawResponse(self._client.account_plans)

    @cached_property
    def aggregations(self) -> aggregations.AggregationsResourceWithRawResponse:
        from .resources.aggregations import AggregationsResourceWithRawResponse

        return AggregationsResourceWithRawResponse(self._client.aggregations)

    @cached_property
    def balances(self) -> balances.BalancesResourceWithRawResponse:
        from .resources.balances import BalancesResourceWithRawResponse

        return BalancesResourceWithRawResponse(self._client.balances)

    @cached_property
    def bills(self) -> bills.BillsResourceWithRawResponse:
        from .resources.bills import BillsResourceWithRawResponse

        return BillsResourceWithRawResponse(self._client.bills)

    @cached_property
    def bill_config(self) -> bill_config.BillConfigResourceWithRawResponse:
        from .resources.bill_config import BillConfigResourceWithRawResponse

        return BillConfigResourceWithRawResponse(self._client.bill_config)

    @cached_property
    def commitments(self) -> commitments.CommitmentsResourceWithRawResponse:
        from .resources.commitments import CommitmentsResourceWithRawResponse

        return CommitmentsResourceWithRawResponse(self._client.commitments)

    @cached_property
    def bill_jobs(self) -> bill_jobs.BillJobsResourceWithRawResponse:
        from .resources.bill_jobs import BillJobsResourceWithRawResponse

        return BillJobsResourceWithRawResponse(self._client.bill_jobs)

    @cached_property
    def charges(self) -> charges.ChargesResourceWithRawResponse:
        from .resources.charges import ChargesResourceWithRawResponse

        return ChargesResourceWithRawResponse(self._client.charges)

    @cached_property
    def compound_aggregations(self) -> compound_aggregations.CompoundAggregationsResourceWithRawResponse:
        from .resources.compound_aggregations import CompoundAggregationsResourceWithRawResponse

        return CompoundAggregationsResourceWithRawResponse(self._client.compound_aggregations)

    @cached_property
    def contracts(self) -> contracts.ContractsResourceWithRawResponse:
        from .resources.contracts import ContractsResourceWithRawResponse

        return ContractsResourceWithRawResponse(self._client.contracts)

    @cached_property
    def counters(self) -> counters.CountersResourceWithRawResponse:
        from .resources.counters import CountersResourceWithRawResponse

        return CountersResourceWithRawResponse(self._client.counters)

    @cached_property
    def counter_adjustments(self) -> counter_adjustments.CounterAdjustmentsResourceWithRawResponse:
        from .resources.counter_adjustments import CounterAdjustmentsResourceWithRawResponse

        return CounterAdjustmentsResourceWithRawResponse(self._client.counter_adjustments)

    @cached_property
    def counter_pricings(self) -> counter_pricings.CounterPricingsResourceWithRawResponse:
        from .resources.counter_pricings import CounterPricingsResourceWithRawResponse

        return CounterPricingsResourceWithRawResponse(self._client.counter_pricings)

    @cached_property
    def credit_reasons(self) -> credit_reasons.CreditReasonsResourceWithRawResponse:
        from .resources.credit_reasons import CreditReasonsResourceWithRawResponse

        return CreditReasonsResourceWithRawResponse(self._client.credit_reasons)

    @cached_property
    def currencies(self) -> currencies.CurrenciesResourceWithRawResponse:
        from .resources.currencies import CurrenciesResourceWithRawResponse

        return CurrenciesResourceWithRawResponse(self._client.currencies)

    @cached_property
    def custom_fields(self) -> custom_fields.CustomFieldsResourceWithRawResponse:
        from .resources.custom_fields import CustomFieldsResourceWithRawResponse

        return CustomFieldsResourceWithRawResponse(self._client.custom_fields)

    @cached_property
    def data_exports(self) -> data_exports.DataExportsResourceWithRawResponse:
        from .resources.data_exports import DataExportsResourceWithRawResponse

        return DataExportsResourceWithRawResponse(self._client.data_exports)

    @cached_property
    def debit_reasons(self) -> debit_reasons.DebitReasonsResourceWithRawResponse:
        from .resources.debit_reasons import DebitReasonsResourceWithRawResponse

        return DebitReasonsResourceWithRawResponse(self._client.debit_reasons)

    @cached_property
    def events(self) -> events.EventsResourceWithRawResponse:
        from .resources.events import EventsResourceWithRawResponse

        return EventsResourceWithRawResponse(self._client.events)

    @cached_property
    def external_mappings(self) -> external_mappings.ExternalMappingsResourceWithRawResponse:
        from .resources.external_mappings import ExternalMappingsResourceWithRawResponse

        return ExternalMappingsResourceWithRawResponse(self._client.external_mappings)

    @cached_property
    def integration_configurations(self) -> integration_configurations.IntegrationConfigurationsResourceWithRawResponse:
        from .resources.integration_configurations import IntegrationConfigurationsResourceWithRawResponse

        return IntegrationConfigurationsResourceWithRawResponse(self._client.integration_configurations)

    @cached_property
    def lookup_tables(self) -> lookup_tables.LookupTablesResourceWithRawResponse:
        from .resources.lookup_tables import LookupTablesResourceWithRawResponse

        return LookupTablesResourceWithRawResponse(self._client.lookup_tables)

    @cached_property
    def meters(self) -> meters.MetersResourceWithRawResponse:
        from .resources.meters import MetersResourceWithRawResponse

        return MetersResourceWithRawResponse(self._client.meters)

    @cached_property
    def notification_configurations(
        self,
    ) -> notification_configurations.NotificationConfigurationsResourceWithRawResponse:
        from .resources.notification_configurations import NotificationConfigurationsResourceWithRawResponse

        return NotificationConfigurationsResourceWithRawResponse(self._client.notification_configurations)

    @cached_property
    def organization_config(self) -> organization_config.OrganizationConfigResourceWithRawResponse:
        from .resources.organization_config import OrganizationConfigResourceWithRawResponse

        return OrganizationConfigResourceWithRawResponse(self._client.organization_config)

    @cached_property
    def permission_policies(self) -> permission_policies.PermissionPoliciesResourceWithRawResponse:
        from .resources.permission_policies import PermissionPoliciesResourceWithRawResponse

        return PermissionPoliciesResourceWithRawResponse(self._client.permission_policies)

    @cached_property
    def plans(self) -> plans.PlansResourceWithRawResponse:
        from .resources.plans import PlansResourceWithRawResponse

        return PlansResourceWithRawResponse(self._client.plans)

    @cached_property
    def plan_groups(self) -> plan_groups.PlanGroupsResourceWithRawResponse:
        from .resources.plan_groups import PlanGroupsResourceWithRawResponse

        return PlanGroupsResourceWithRawResponse(self._client.plan_groups)

    @cached_property
    def plan_group_links(self) -> plan_group_links.PlanGroupLinksResourceWithRawResponse:
        from .resources.plan_group_links import PlanGroupLinksResourceWithRawResponse

        return PlanGroupLinksResourceWithRawResponse(self._client.plan_group_links)

    @cached_property
    def plan_templates(self) -> plan_templates.PlanTemplatesResourceWithRawResponse:
        from .resources.plan_templates import PlanTemplatesResourceWithRawResponse

        return PlanTemplatesResourceWithRawResponse(self._client.plan_templates)

    @cached_property
    def pricings(self) -> pricings.PricingsResourceWithRawResponse:
        from .resources.pricings import PricingsResourceWithRawResponse

        return PricingsResourceWithRawResponse(self._client.pricings)

    @cached_property
    def products(self) -> products.ProductsResourceWithRawResponse:
        from .resources.products import ProductsResourceWithRawResponse

        return ProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def resource_groups(self) -> resource_groups.ResourceGroupsResourceWithRawResponse:
        from .resources.resource_groups import ResourceGroupsResourceWithRawResponse

        return ResourceGroupsResourceWithRawResponse(self._client.resource_groups)

    @cached_property
    def scheduled_event_configurations(
        self,
    ) -> scheduled_event_configurations.ScheduledEventConfigurationsResourceWithRawResponse:
        from .resources.scheduled_event_configurations import ScheduledEventConfigurationsResourceWithRawResponse

        return ScheduledEventConfigurationsResourceWithRawResponse(self._client.scheduled_event_configurations)

    @cached_property
    def statements(self) -> statements.StatementsResourceWithRawResponse:
        from .resources.statements import StatementsResourceWithRawResponse

        return StatementsResourceWithRawResponse(self._client.statements)

    @cached_property
    def transaction_types(self) -> transaction_types.TransactionTypesResourceWithRawResponse:
        from .resources.transaction_types import TransactionTypesResourceWithRawResponse

        return TransactionTypesResourceWithRawResponse(self._client.transaction_types)

    @cached_property
    def usage(self) -> usage.UsageResourceWithRawResponse:
        from .resources.usage import UsageResourceWithRawResponse

        return UsageResourceWithRawResponse(self._client.usage)

    @cached_property
    def users(self) -> users.UsersResourceWithRawResponse:
        from .resources.users import UsersResourceWithRawResponse

        return UsersResourceWithRawResponse(self._client.users)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithRawResponse:
        from .resources.webhooks import WebhooksResourceWithRawResponse

        return WebhooksResourceWithRawResponse(self._client.webhooks)


class AsyncM3terWithRawResponse:
    _client: AsyncM3ter

    def __init__(self, client: AsyncM3ter) -> None:
        self._client = client

    @cached_property
    def authentication(self) -> authentication.AsyncAuthenticationResourceWithRawResponse:
        from .resources.authentication import AsyncAuthenticationResourceWithRawResponse

        return AsyncAuthenticationResourceWithRawResponse(self._client.authentication)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithRawResponse:
        from .resources.accounts import AsyncAccountsResourceWithRawResponse

        return AsyncAccountsResourceWithRawResponse(self._client.accounts)

    @cached_property
    def account_plans(self) -> account_plans.AsyncAccountPlansResourceWithRawResponse:
        from .resources.account_plans import AsyncAccountPlansResourceWithRawResponse

        return AsyncAccountPlansResourceWithRawResponse(self._client.account_plans)

    @cached_property
    def aggregations(self) -> aggregations.AsyncAggregationsResourceWithRawResponse:
        from .resources.aggregations import AsyncAggregationsResourceWithRawResponse

        return AsyncAggregationsResourceWithRawResponse(self._client.aggregations)

    @cached_property
    def balances(self) -> balances.AsyncBalancesResourceWithRawResponse:
        from .resources.balances import AsyncBalancesResourceWithRawResponse

        return AsyncBalancesResourceWithRawResponse(self._client.balances)

    @cached_property
    def bills(self) -> bills.AsyncBillsResourceWithRawResponse:
        from .resources.bills import AsyncBillsResourceWithRawResponse

        return AsyncBillsResourceWithRawResponse(self._client.bills)

    @cached_property
    def bill_config(self) -> bill_config.AsyncBillConfigResourceWithRawResponse:
        from .resources.bill_config import AsyncBillConfigResourceWithRawResponse

        return AsyncBillConfigResourceWithRawResponse(self._client.bill_config)

    @cached_property
    def commitments(self) -> commitments.AsyncCommitmentsResourceWithRawResponse:
        from .resources.commitments import AsyncCommitmentsResourceWithRawResponse

        return AsyncCommitmentsResourceWithRawResponse(self._client.commitments)

    @cached_property
    def bill_jobs(self) -> bill_jobs.AsyncBillJobsResourceWithRawResponse:
        from .resources.bill_jobs import AsyncBillJobsResourceWithRawResponse

        return AsyncBillJobsResourceWithRawResponse(self._client.bill_jobs)

    @cached_property
    def charges(self) -> charges.AsyncChargesResourceWithRawResponse:
        from .resources.charges import AsyncChargesResourceWithRawResponse

        return AsyncChargesResourceWithRawResponse(self._client.charges)

    @cached_property
    def compound_aggregations(self) -> compound_aggregations.AsyncCompoundAggregationsResourceWithRawResponse:
        from .resources.compound_aggregations import AsyncCompoundAggregationsResourceWithRawResponse

        return AsyncCompoundAggregationsResourceWithRawResponse(self._client.compound_aggregations)

    @cached_property
    def contracts(self) -> contracts.AsyncContractsResourceWithRawResponse:
        from .resources.contracts import AsyncContractsResourceWithRawResponse

        return AsyncContractsResourceWithRawResponse(self._client.contracts)

    @cached_property
    def counters(self) -> counters.AsyncCountersResourceWithRawResponse:
        from .resources.counters import AsyncCountersResourceWithRawResponse

        return AsyncCountersResourceWithRawResponse(self._client.counters)

    @cached_property
    def counter_adjustments(self) -> counter_adjustments.AsyncCounterAdjustmentsResourceWithRawResponse:
        from .resources.counter_adjustments import AsyncCounterAdjustmentsResourceWithRawResponse

        return AsyncCounterAdjustmentsResourceWithRawResponse(self._client.counter_adjustments)

    @cached_property
    def counter_pricings(self) -> counter_pricings.AsyncCounterPricingsResourceWithRawResponse:
        from .resources.counter_pricings import AsyncCounterPricingsResourceWithRawResponse

        return AsyncCounterPricingsResourceWithRawResponse(self._client.counter_pricings)

    @cached_property
    def credit_reasons(self) -> credit_reasons.AsyncCreditReasonsResourceWithRawResponse:
        from .resources.credit_reasons import AsyncCreditReasonsResourceWithRawResponse

        return AsyncCreditReasonsResourceWithRawResponse(self._client.credit_reasons)

    @cached_property
    def currencies(self) -> currencies.AsyncCurrenciesResourceWithRawResponse:
        from .resources.currencies import AsyncCurrenciesResourceWithRawResponse

        return AsyncCurrenciesResourceWithRawResponse(self._client.currencies)

    @cached_property
    def custom_fields(self) -> custom_fields.AsyncCustomFieldsResourceWithRawResponse:
        from .resources.custom_fields import AsyncCustomFieldsResourceWithRawResponse

        return AsyncCustomFieldsResourceWithRawResponse(self._client.custom_fields)

    @cached_property
    def data_exports(self) -> data_exports.AsyncDataExportsResourceWithRawResponse:
        from .resources.data_exports import AsyncDataExportsResourceWithRawResponse

        return AsyncDataExportsResourceWithRawResponse(self._client.data_exports)

    @cached_property
    def debit_reasons(self) -> debit_reasons.AsyncDebitReasonsResourceWithRawResponse:
        from .resources.debit_reasons import AsyncDebitReasonsResourceWithRawResponse

        return AsyncDebitReasonsResourceWithRawResponse(self._client.debit_reasons)

    @cached_property
    def events(self) -> events.AsyncEventsResourceWithRawResponse:
        from .resources.events import AsyncEventsResourceWithRawResponse

        return AsyncEventsResourceWithRawResponse(self._client.events)

    @cached_property
    def external_mappings(self) -> external_mappings.AsyncExternalMappingsResourceWithRawResponse:
        from .resources.external_mappings import AsyncExternalMappingsResourceWithRawResponse

        return AsyncExternalMappingsResourceWithRawResponse(self._client.external_mappings)

    @cached_property
    def integration_configurations(
        self,
    ) -> integration_configurations.AsyncIntegrationConfigurationsResourceWithRawResponse:
        from .resources.integration_configurations import AsyncIntegrationConfigurationsResourceWithRawResponse

        return AsyncIntegrationConfigurationsResourceWithRawResponse(self._client.integration_configurations)

    @cached_property
    def lookup_tables(self) -> lookup_tables.AsyncLookupTablesResourceWithRawResponse:
        from .resources.lookup_tables import AsyncLookupTablesResourceWithRawResponse

        return AsyncLookupTablesResourceWithRawResponse(self._client.lookup_tables)

    @cached_property
    def meters(self) -> meters.AsyncMetersResourceWithRawResponse:
        from .resources.meters import AsyncMetersResourceWithRawResponse

        return AsyncMetersResourceWithRawResponse(self._client.meters)

    @cached_property
    def notification_configurations(
        self,
    ) -> notification_configurations.AsyncNotificationConfigurationsResourceWithRawResponse:
        from .resources.notification_configurations import AsyncNotificationConfigurationsResourceWithRawResponse

        return AsyncNotificationConfigurationsResourceWithRawResponse(self._client.notification_configurations)

    @cached_property
    def organization_config(self) -> organization_config.AsyncOrganizationConfigResourceWithRawResponse:
        from .resources.organization_config import AsyncOrganizationConfigResourceWithRawResponse

        return AsyncOrganizationConfigResourceWithRawResponse(self._client.organization_config)

    @cached_property
    def permission_policies(self) -> permission_policies.AsyncPermissionPoliciesResourceWithRawResponse:
        from .resources.permission_policies import AsyncPermissionPoliciesResourceWithRawResponse

        return AsyncPermissionPoliciesResourceWithRawResponse(self._client.permission_policies)

    @cached_property
    def plans(self) -> plans.AsyncPlansResourceWithRawResponse:
        from .resources.plans import AsyncPlansResourceWithRawResponse

        return AsyncPlansResourceWithRawResponse(self._client.plans)

    @cached_property
    def plan_groups(self) -> plan_groups.AsyncPlanGroupsResourceWithRawResponse:
        from .resources.plan_groups import AsyncPlanGroupsResourceWithRawResponse

        return AsyncPlanGroupsResourceWithRawResponse(self._client.plan_groups)

    @cached_property
    def plan_group_links(self) -> plan_group_links.AsyncPlanGroupLinksResourceWithRawResponse:
        from .resources.plan_group_links import AsyncPlanGroupLinksResourceWithRawResponse

        return AsyncPlanGroupLinksResourceWithRawResponse(self._client.plan_group_links)

    @cached_property
    def plan_templates(self) -> plan_templates.AsyncPlanTemplatesResourceWithRawResponse:
        from .resources.plan_templates import AsyncPlanTemplatesResourceWithRawResponse

        return AsyncPlanTemplatesResourceWithRawResponse(self._client.plan_templates)

    @cached_property
    def pricings(self) -> pricings.AsyncPricingsResourceWithRawResponse:
        from .resources.pricings import AsyncPricingsResourceWithRawResponse

        return AsyncPricingsResourceWithRawResponse(self._client.pricings)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithRawResponse:
        from .resources.products import AsyncProductsResourceWithRawResponse

        return AsyncProductsResourceWithRawResponse(self._client.products)

    @cached_property
    def resource_groups(self) -> resource_groups.AsyncResourceGroupsResourceWithRawResponse:
        from .resources.resource_groups import AsyncResourceGroupsResourceWithRawResponse

        return AsyncResourceGroupsResourceWithRawResponse(self._client.resource_groups)

    @cached_property
    def scheduled_event_configurations(
        self,
    ) -> scheduled_event_configurations.AsyncScheduledEventConfigurationsResourceWithRawResponse:
        from .resources.scheduled_event_configurations import AsyncScheduledEventConfigurationsResourceWithRawResponse

        return AsyncScheduledEventConfigurationsResourceWithRawResponse(self._client.scheduled_event_configurations)

    @cached_property
    def statements(self) -> statements.AsyncStatementsResourceWithRawResponse:
        from .resources.statements import AsyncStatementsResourceWithRawResponse

        return AsyncStatementsResourceWithRawResponse(self._client.statements)

    @cached_property
    def transaction_types(self) -> transaction_types.AsyncTransactionTypesResourceWithRawResponse:
        from .resources.transaction_types import AsyncTransactionTypesResourceWithRawResponse

        return AsyncTransactionTypesResourceWithRawResponse(self._client.transaction_types)

    @cached_property
    def usage(self) -> usage.AsyncUsageResourceWithRawResponse:
        from .resources.usage import AsyncUsageResourceWithRawResponse

        return AsyncUsageResourceWithRawResponse(self._client.usage)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithRawResponse:
        from .resources.users import AsyncUsersResourceWithRawResponse

        return AsyncUsersResourceWithRawResponse(self._client.users)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithRawResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithRawResponse

        return AsyncWebhooksResourceWithRawResponse(self._client.webhooks)


class M3terWithStreamedResponse:
    _client: M3ter

    def __init__(self, client: M3ter) -> None:
        self._client = client

    @cached_property
    def authentication(self) -> authentication.AuthenticationResourceWithStreamingResponse:
        from .resources.authentication import AuthenticationResourceWithStreamingResponse

        return AuthenticationResourceWithStreamingResponse(self._client.authentication)

    @cached_property
    def accounts(self) -> accounts.AccountsResourceWithStreamingResponse:
        from .resources.accounts import AccountsResourceWithStreamingResponse

        return AccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def account_plans(self) -> account_plans.AccountPlansResourceWithStreamingResponse:
        from .resources.account_plans import AccountPlansResourceWithStreamingResponse

        return AccountPlansResourceWithStreamingResponse(self._client.account_plans)

    @cached_property
    def aggregations(self) -> aggregations.AggregationsResourceWithStreamingResponse:
        from .resources.aggregations import AggregationsResourceWithStreamingResponse

        return AggregationsResourceWithStreamingResponse(self._client.aggregations)

    @cached_property
    def balances(self) -> balances.BalancesResourceWithStreamingResponse:
        from .resources.balances import BalancesResourceWithStreamingResponse

        return BalancesResourceWithStreamingResponse(self._client.balances)

    @cached_property
    def bills(self) -> bills.BillsResourceWithStreamingResponse:
        from .resources.bills import BillsResourceWithStreamingResponse

        return BillsResourceWithStreamingResponse(self._client.bills)

    @cached_property
    def bill_config(self) -> bill_config.BillConfigResourceWithStreamingResponse:
        from .resources.bill_config import BillConfigResourceWithStreamingResponse

        return BillConfigResourceWithStreamingResponse(self._client.bill_config)

    @cached_property
    def commitments(self) -> commitments.CommitmentsResourceWithStreamingResponse:
        from .resources.commitments import CommitmentsResourceWithStreamingResponse

        return CommitmentsResourceWithStreamingResponse(self._client.commitments)

    @cached_property
    def bill_jobs(self) -> bill_jobs.BillJobsResourceWithStreamingResponse:
        from .resources.bill_jobs import BillJobsResourceWithStreamingResponse

        return BillJobsResourceWithStreamingResponse(self._client.bill_jobs)

    @cached_property
    def charges(self) -> charges.ChargesResourceWithStreamingResponse:
        from .resources.charges import ChargesResourceWithStreamingResponse

        return ChargesResourceWithStreamingResponse(self._client.charges)

    @cached_property
    def compound_aggregations(self) -> compound_aggregations.CompoundAggregationsResourceWithStreamingResponse:
        from .resources.compound_aggregations import CompoundAggregationsResourceWithStreamingResponse

        return CompoundAggregationsResourceWithStreamingResponse(self._client.compound_aggregations)

    @cached_property
    def contracts(self) -> contracts.ContractsResourceWithStreamingResponse:
        from .resources.contracts import ContractsResourceWithStreamingResponse

        return ContractsResourceWithStreamingResponse(self._client.contracts)

    @cached_property
    def counters(self) -> counters.CountersResourceWithStreamingResponse:
        from .resources.counters import CountersResourceWithStreamingResponse

        return CountersResourceWithStreamingResponse(self._client.counters)

    @cached_property
    def counter_adjustments(self) -> counter_adjustments.CounterAdjustmentsResourceWithStreamingResponse:
        from .resources.counter_adjustments import CounterAdjustmentsResourceWithStreamingResponse

        return CounterAdjustmentsResourceWithStreamingResponse(self._client.counter_adjustments)

    @cached_property
    def counter_pricings(self) -> counter_pricings.CounterPricingsResourceWithStreamingResponse:
        from .resources.counter_pricings import CounterPricingsResourceWithStreamingResponse

        return CounterPricingsResourceWithStreamingResponse(self._client.counter_pricings)

    @cached_property
    def credit_reasons(self) -> credit_reasons.CreditReasonsResourceWithStreamingResponse:
        from .resources.credit_reasons import CreditReasonsResourceWithStreamingResponse

        return CreditReasonsResourceWithStreamingResponse(self._client.credit_reasons)

    @cached_property
    def currencies(self) -> currencies.CurrenciesResourceWithStreamingResponse:
        from .resources.currencies import CurrenciesResourceWithStreamingResponse

        return CurrenciesResourceWithStreamingResponse(self._client.currencies)

    @cached_property
    def custom_fields(self) -> custom_fields.CustomFieldsResourceWithStreamingResponse:
        from .resources.custom_fields import CustomFieldsResourceWithStreamingResponse

        return CustomFieldsResourceWithStreamingResponse(self._client.custom_fields)

    @cached_property
    def data_exports(self) -> data_exports.DataExportsResourceWithStreamingResponse:
        from .resources.data_exports import DataExportsResourceWithStreamingResponse

        return DataExportsResourceWithStreamingResponse(self._client.data_exports)

    @cached_property
    def debit_reasons(self) -> debit_reasons.DebitReasonsResourceWithStreamingResponse:
        from .resources.debit_reasons import DebitReasonsResourceWithStreamingResponse

        return DebitReasonsResourceWithStreamingResponse(self._client.debit_reasons)

    @cached_property
    def events(self) -> events.EventsResourceWithStreamingResponse:
        from .resources.events import EventsResourceWithStreamingResponse

        return EventsResourceWithStreamingResponse(self._client.events)

    @cached_property
    def external_mappings(self) -> external_mappings.ExternalMappingsResourceWithStreamingResponse:
        from .resources.external_mappings import ExternalMappingsResourceWithStreamingResponse

        return ExternalMappingsResourceWithStreamingResponse(self._client.external_mappings)

    @cached_property
    def integration_configurations(
        self,
    ) -> integration_configurations.IntegrationConfigurationsResourceWithStreamingResponse:
        from .resources.integration_configurations import IntegrationConfigurationsResourceWithStreamingResponse

        return IntegrationConfigurationsResourceWithStreamingResponse(self._client.integration_configurations)

    @cached_property
    def lookup_tables(self) -> lookup_tables.LookupTablesResourceWithStreamingResponse:
        from .resources.lookup_tables import LookupTablesResourceWithStreamingResponse

        return LookupTablesResourceWithStreamingResponse(self._client.lookup_tables)

    @cached_property
    def meters(self) -> meters.MetersResourceWithStreamingResponse:
        from .resources.meters import MetersResourceWithStreamingResponse

        return MetersResourceWithStreamingResponse(self._client.meters)

    @cached_property
    def notification_configurations(
        self,
    ) -> notification_configurations.NotificationConfigurationsResourceWithStreamingResponse:
        from .resources.notification_configurations import NotificationConfigurationsResourceWithStreamingResponse

        return NotificationConfigurationsResourceWithStreamingResponse(self._client.notification_configurations)

    @cached_property
    def organization_config(self) -> organization_config.OrganizationConfigResourceWithStreamingResponse:
        from .resources.organization_config import OrganizationConfigResourceWithStreamingResponse

        return OrganizationConfigResourceWithStreamingResponse(self._client.organization_config)

    @cached_property
    def permission_policies(self) -> permission_policies.PermissionPoliciesResourceWithStreamingResponse:
        from .resources.permission_policies import PermissionPoliciesResourceWithStreamingResponse

        return PermissionPoliciesResourceWithStreamingResponse(self._client.permission_policies)

    @cached_property
    def plans(self) -> plans.PlansResourceWithStreamingResponse:
        from .resources.plans import PlansResourceWithStreamingResponse

        return PlansResourceWithStreamingResponse(self._client.plans)

    @cached_property
    def plan_groups(self) -> plan_groups.PlanGroupsResourceWithStreamingResponse:
        from .resources.plan_groups import PlanGroupsResourceWithStreamingResponse

        return PlanGroupsResourceWithStreamingResponse(self._client.plan_groups)

    @cached_property
    def plan_group_links(self) -> plan_group_links.PlanGroupLinksResourceWithStreamingResponse:
        from .resources.plan_group_links import PlanGroupLinksResourceWithStreamingResponse

        return PlanGroupLinksResourceWithStreamingResponse(self._client.plan_group_links)

    @cached_property
    def plan_templates(self) -> plan_templates.PlanTemplatesResourceWithStreamingResponse:
        from .resources.plan_templates import PlanTemplatesResourceWithStreamingResponse

        return PlanTemplatesResourceWithStreamingResponse(self._client.plan_templates)

    @cached_property
    def pricings(self) -> pricings.PricingsResourceWithStreamingResponse:
        from .resources.pricings import PricingsResourceWithStreamingResponse

        return PricingsResourceWithStreamingResponse(self._client.pricings)

    @cached_property
    def products(self) -> products.ProductsResourceWithStreamingResponse:
        from .resources.products import ProductsResourceWithStreamingResponse

        return ProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def resource_groups(self) -> resource_groups.ResourceGroupsResourceWithStreamingResponse:
        from .resources.resource_groups import ResourceGroupsResourceWithStreamingResponse

        return ResourceGroupsResourceWithStreamingResponse(self._client.resource_groups)

    @cached_property
    def scheduled_event_configurations(
        self,
    ) -> scheduled_event_configurations.ScheduledEventConfigurationsResourceWithStreamingResponse:
        from .resources.scheduled_event_configurations import ScheduledEventConfigurationsResourceWithStreamingResponse

        return ScheduledEventConfigurationsResourceWithStreamingResponse(self._client.scheduled_event_configurations)

    @cached_property
    def statements(self) -> statements.StatementsResourceWithStreamingResponse:
        from .resources.statements import StatementsResourceWithStreamingResponse

        return StatementsResourceWithStreamingResponse(self._client.statements)

    @cached_property
    def transaction_types(self) -> transaction_types.TransactionTypesResourceWithStreamingResponse:
        from .resources.transaction_types import TransactionTypesResourceWithStreamingResponse

        return TransactionTypesResourceWithStreamingResponse(self._client.transaction_types)

    @cached_property
    def usage(self) -> usage.UsageResourceWithStreamingResponse:
        from .resources.usage import UsageResourceWithStreamingResponse

        return UsageResourceWithStreamingResponse(self._client.usage)

    @cached_property
    def users(self) -> users.UsersResourceWithStreamingResponse:
        from .resources.users import UsersResourceWithStreamingResponse

        return UsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def webhooks(self) -> webhooks.WebhooksResourceWithStreamingResponse:
        from .resources.webhooks import WebhooksResourceWithStreamingResponse

        return WebhooksResourceWithStreamingResponse(self._client.webhooks)


class AsyncM3terWithStreamedResponse:
    _client: AsyncM3ter

    def __init__(self, client: AsyncM3ter) -> None:
        self._client = client

    @cached_property
    def authentication(self) -> authentication.AsyncAuthenticationResourceWithStreamingResponse:
        from .resources.authentication import AsyncAuthenticationResourceWithStreamingResponse

        return AsyncAuthenticationResourceWithStreamingResponse(self._client.authentication)

    @cached_property
    def accounts(self) -> accounts.AsyncAccountsResourceWithStreamingResponse:
        from .resources.accounts import AsyncAccountsResourceWithStreamingResponse

        return AsyncAccountsResourceWithStreamingResponse(self._client.accounts)

    @cached_property
    def account_plans(self) -> account_plans.AsyncAccountPlansResourceWithStreamingResponse:
        from .resources.account_plans import AsyncAccountPlansResourceWithStreamingResponse

        return AsyncAccountPlansResourceWithStreamingResponse(self._client.account_plans)

    @cached_property
    def aggregations(self) -> aggregations.AsyncAggregationsResourceWithStreamingResponse:
        from .resources.aggregations import AsyncAggregationsResourceWithStreamingResponse

        return AsyncAggregationsResourceWithStreamingResponse(self._client.aggregations)

    @cached_property
    def balances(self) -> balances.AsyncBalancesResourceWithStreamingResponse:
        from .resources.balances import AsyncBalancesResourceWithStreamingResponse

        return AsyncBalancesResourceWithStreamingResponse(self._client.balances)

    @cached_property
    def bills(self) -> bills.AsyncBillsResourceWithStreamingResponse:
        from .resources.bills import AsyncBillsResourceWithStreamingResponse

        return AsyncBillsResourceWithStreamingResponse(self._client.bills)

    @cached_property
    def bill_config(self) -> bill_config.AsyncBillConfigResourceWithStreamingResponse:
        from .resources.bill_config import AsyncBillConfigResourceWithStreamingResponse

        return AsyncBillConfigResourceWithStreamingResponse(self._client.bill_config)

    @cached_property
    def commitments(self) -> commitments.AsyncCommitmentsResourceWithStreamingResponse:
        from .resources.commitments import AsyncCommitmentsResourceWithStreamingResponse

        return AsyncCommitmentsResourceWithStreamingResponse(self._client.commitments)

    @cached_property
    def bill_jobs(self) -> bill_jobs.AsyncBillJobsResourceWithStreamingResponse:
        from .resources.bill_jobs import AsyncBillJobsResourceWithStreamingResponse

        return AsyncBillJobsResourceWithStreamingResponse(self._client.bill_jobs)

    @cached_property
    def charges(self) -> charges.AsyncChargesResourceWithStreamingResponse:
        from .resources.charges import AsyncChargesResourceWithStreamingResponse

        return AsyncChargesResourceWithStreamingResponse(self._client.charges)

    @cached_property
    def compound_aggregations(self) -> compound_aggregations.AsyncCompoundAggregationsResourceWithStreamingResponse:
        from .resources.compound_aggregations import AsyncCompoundAggregationsResourceWithStreamingResponse

        return AsyncCompoundAggregationsResourceWithStreamingResponse(self._client.compound_aggregations)

    @cached_property
    def contracts(self) -> contracts.AsyncContractsResourceWithStreamingResponse:
        from .resources.contracts import AsyncContractsResourceWithStreamingResponse

        return AsyncContractsResourceWithStreamingResponse(self._client.contracts)

    @cached_property
    def counters(self) -> counters.AsyncCountersResourceWithStreamingResponse:
        from .resources.counters import AsyncCountersResourceWithStreamingResponse

        return AsyncCountersResourceWithStreamingResponse(self._client.counters)

    @cached_property
    def counter_adjustments(self) -> counter_adjustments.AsyncCounterAdjustmentsResourceWithStreamingResponse:
        from .resources.counter_adjustments import AsyncCounterAdjustmentsResourceWithStreamingResponse

        return AsyncCounterAdjustmentsResourceWithStreamingResponse(self._client.counter_adjustments)

    @cached_property
    def counter_pricings(self) -> counter_pricings.AsyncCounterPricingsResourceWithStreamingResponse:
        from .resources.counter_pricings import AsyncCounterPricingsResourceWithStreamingResponse

        return AsyncCounterPricingsResourceWithStreamingResponse(self._client.counter_pricings)

    @cached_property
    def credit_reasons(self) -> credit_reasons.AsyncCreditReasonsResourceWithStreamingResponse:
        from .resources.credit_reasons import AsyncCreditReasonsResourceWithStreamingResponse

        return AsyncCreditReasonsResourceWithStreamingResponse(self._client.credit_reasons)

    @cached_property
    def currencies(self) -> currencies.AsyncCurrenciesResourceWithStreamingResponse:
        from .resources.currencies import AsyncCurrenciesResourceWithStreamingResponse

        return AsyncCurrenciesResourceWithStreamingResponse(self._client.currencies)

    @cached_property
    def custom_fields(self) -> custom_fields.AsyncCustomFieldsResourceWithStreamingResponse:
        from .resources.custom_fields import AsyncCustomFieldsResourceWithStreamingResponse

        return AsyncCustomFieldsResourceWithStreamingResponse(self._client.custom_fields)

    @cached_property
    def data_exports(self) -> data_exports.AsyncDataExportsResourceWithStreamingResponse:
        from .resources.data_exports import AsyncDataExportsResourceWithStreamingResponse

        return AsyncDataExportsResourceWithStreamingResponse(self._client.data_exports)

    @cached_property
    def debit_reasons(self) -> debit_reasons.AsyncDebitReasonsResourceWithStreamingResponse:
        from .resources.debit_reasons import AsyncDebitReasonsResourceWithStreamingResponse

        return AsyncDebitReasonsResourceWithStreamingResponse(self._client.debit_reasons)

    @cached_property
    def events(self) -> events.AsyncEventsResourceWithStreamingResponse:
        from .resources.events import AsyncEventsResourceWithStreamingResponse

        return AsyncEventsResourceWithStreamingResponse(self._client.events)

    @cached_property
    def external_mappings(self) -> external_mappings.AsyncExternalMappingsResourceWithStreamingResponse:
        from .resources.external_mappings import AsyncExternalMappingsResourceWithStreamingResponse

        return AsyncExternalMappingsResourceWithStreamingResponse(self._client.external_mappings)

    @cached_property
    def integration_configurations(
        self,
    ) -> integration_configurations.AsyncIntegrationConfigurationsResourceWithStreamingResponse:
        from .resources.integration_configurations import AsyncIntegrationConfigurationsResourceWithStreamingResponse

        return AsyncIntegrationConfigurationsResourceWithStreamingResponse(self._client.integration_configurations)

    @cached_property
    def lookup_tables(self) -> lookup_tables.AsyncLookupTablesResourceWithStreamingResponse:
        from .resources.lookup_tables import AsyncLookupTablesResourceWithStreamingResponse

        return AsyncLookupTablesResourceWithStreamingResponse(self._client.lookup_tables)

    @cached_property
    def meters(self) -> meters.AsyncMetersResourceWithStreamingResponse:
        from .resources.meters import AsyncMetersResourceWithStreamingResponse

        return AsyncMetersResourceWithStreamingResponse(self._client.meters)

    @cached_property
    def notification_configurations(
        self,
    ) -> notification_configurations.AsyncNotificationConfigurationsResourceWithStreamingResponse:
        from .resources.notification_configurations import AsyncNotificationConfigurationsResourceWithStreamingResponse

        return AsyncNotificationConfigurationsResourceWithStreamingResponse(self._client.notification_configurations)

    @cached_property
    def organization_config(self) -> organization_config.AsyncOrganizationConfigResourceWithStreamingResponse:
        from .resources.organization_config import AsyncOrganizationConfigResourceWithStreamingResponse

        return AsyncOrganizationConfigResourceWithStreamingResponse(self._client.organization_config)

    @cached_property
    def permission_policies(self) -> permission_policies.AsyncPermissionPoliciesResourceWithStreamingResponse:
        from .resources.permission_policies import AsyncPermissionPoliciesResourceWithStreamingResponse

        return AsyncPermissionPoliciesResourceWithStreamingResponse(self._client.permission_policies)

    @cached_property
    def plans(self) -> plans.AsyncPlansResourceWithStreamingResponse:
        from .resources.plans import AsyncPlansResourceWithStreamingResponse

        return AsyncPlansResourceWithStreamingResponse(self._client.plans)

    @cached_property
    def plan_groups(self) -> plan_groups.AsyncPlanGroupsResourceWithStreamingResponse:
        from .resources.plan_groups import AsyncPlanGroupsResourceWithStreamingResponse

        return AsyncPlanGroupsResourceWithStreamingResponse(self._client.plan_groups)

    @cached_property
    def plan_group_links(self) -> plan_group_links.AsyncPlanGroupLinksResourceWithStreamingResponse:
        from .resources.plan_group_links import AsyncPlanGroupLinksResourceWithStreamingResponse

        return AsyncPlanGroupLinksResourceWithStreamingResponse(self._client.plan_group_links)

    @cached_property
    def plan_templates(self) -> plan_templates.AsyncPlanTemplatesResourceWithStreamingResponse:
        from .resources.plan_templates import AsyncPlanTemplatesResourceWithStreamingResponse

        return AsyncPlanTemplatesResourceWithStreamingResponse(self._client.plan_templates)

    @cached_property
    def pricings(self) -> pricings.AsyncPricingsResourceWithStreamingResponse:
        from .resources.pricings import AsyncPricingsResourceWithStreamingResponse

        return AsyncPricingsResourceWithStreamingResponse(self._client.pricings)

    @cached_property
    def products(self) -> products.AsyncProductsResourceWithStreamingResponse:
        from .resources.products import AsyncProductsResourceWithStreamingResponse

        return AsyncProductsResourceWithStreamingResponse(self._client.products)

    @cached_property
    def resource_groups(self) -> resource_groups.AsyncResourceGroupsResourceWithStreamingResponse:
        from .resources.resource_groups import AsyncResourceGroupsResourceWithStreamingResponse

        return AsyncResourceGroupsResourceWithStreamingResponse(self._client.resource_groups)

    @cached_property
    def scheduled_event_configurations(
        self,
    ) -> scheduled_event_configurations.AsyncScheduledEventConfigurationsResourceWithStreamingResponse:
        from .resources.scheduled_event_configurations import (
            AsyncScheduledEventConfigurationsResourceWithStreamingResponse,
        )

        return AsyncScheduledEventConfigurationsResourceWithStreamingResponse(
            self._client.scheduled_event_configurations
        )

    @cached_property
    def statements(self) -> statements.AsyncStatementsResourceWithStreamingResponse:
        from .resources.statements import AsyncStatementsResourceWithStreamingResponse

        return AsyncStatementsResourceWithStreamingResponse(self._client.statements)

    @cached_property
    def transaction_types(self) -> transaction_types.AsyncTransactionTypesResourceWithStreamingResponse:
        from .resources.transaction_types import AsyncTransactionTypesResourceWithStreamingResponse

        return AsyncTransactionTypesResourceWithStreamingResponse(self._client.transaction_types)

    @cached_property
    def usage(self) -> usage.AsyncUsageResourceWithStreamingResponse:
        from .resources.usage import AsyncUsageResourceWithStreamingResponse

        return AsyncUsageResourceWithStreamingResponse(self._client.usage)

    @cached_property
    def users(self) -> users.AsyncUsersResourceWithStreamingResponse:
        from .resources.users import AsyncUsersResourceWithStreamingResponse

        return AsyncUsersResourceWithStreamingResponse(self._client.users)

    @cached_property
    def webhooks(self) -> webhooks.AsyncWebhooksResourceWithStreamingResponse:
        from .resources.webhooks import AsyncWebhooksResourceWithStreamingResponse

        return AsyncWebhooksResourceWithStreamingResponse(self._client.webhooks)


Client = M3ter

AsyncClient = AsyncM3ter



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .schedule_response import ScheduleResponse as ScheduleResponse
from .transaction_response import TransactionResponse as TransactionResponse
from .transaction_list_params import TransactionListParams as TransactionListParams
from .transaction_create_params import TransactionCreateParams as TransactionCreateParams
from .charge_schedule_list_params import ChargeScheduleListParams as ChargeScheduleListParams
from .transaction_summary_response import TransactionSummaryResponse as TransactionSummaryResponse
from .charge_schedule_create_params import ChargeScheduleCreateParams as ChargeScheduleCreateParams
from .charge_schedule_list_response import ChargeScheduleListResponse as ChargeScheduleListResponse
from .charge_schedule_update_params import ChargeScheduleUpdateParams as ChargeScheduleUpdateParams
from .charge_schedule_preview_params import ChargeSchedulePreviewParams as ChargeSchedulePreviewParams
from .charge_schedule_create_response import ChargeScheduleCreateResponse as ChargeScheduleCreateResponse
from .charge_schedule_delete_response import ChargeScheduleDeleteResponse as ChargeScheduleDeleteResponse
from .charge_schedule_update_response import ChargeScheduleUpdateResponse as ChargeScheduleUpdateResponse
from .charge_schedule_preview_response import ChargeSchedulePreviewResponse as ChargeSchedulePreviewResponse
from .transaction_schedule_list_params import TransactionScheduleListParams as TransactionScheduleListParams
from .charge_schedule_retrieve_response import ChargeScheduleRetrieveResponse as ChargeScheduleRetrieveResponse
from .transaction_schedule_create_params import TransactionScheduleCreateParams as TransactionScheduleCreateParams
from .transaction_schedule_update_params import TransactionScheduleUpdateParams as TransactionScheduleUpdateParams
from .transaction_schedule_preview_params import TransactionSchedulePreviewParams as TransactionSchedulePreviewParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, overload

import httpx

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from ...types import data_export_create_adhoc_params
from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import required_args, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from .schedules import (
    SchedulesResource,
    AsyncSchedulesResource,
    SchedulesResourceWithRawResponse,
    AsyncSchedulesResourceWithRawResponse,
    SchedulesResourceWithStreamingResponse,
    AsyncSchedulesResourceWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .destinations import (
    DestinationsResource,
    AsyncDestinationsResource,
    DestinationsResourceWithRawResponse,
    AsyncDestinationsResourceWithRawResponse,
    DestinationsResourceWithStreamingResponse,
    AsyncDestinationsResourceWithStreamingResponse,
)
from ..._base_client import make_request_options
from ...types.ad_hoc_response import AdHocResponse
from ...types.data_explorer_group_param import DataExplorerGroupParam

__all__ = ["DataExportsResource", "AsyncDataExportsResource"]


class DataExportsResource(SyncAPIResource):
    @cached_property
    def destinations(self) -> DestinationsResource:
        return DestinationsResource(self._client)

    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def schedules(self) -> SchedulesResource:
        return SchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DataExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return DataExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DataExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return DataExportsResourceWithStreamingResponse(self)

    @overload
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
                "TRANSACTION_TYPES",
                "CHARGES",
            ]
        ],
        source_type: Literal["OPERATIONAL"],
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          ad-hoc export.
        - You can use the `dimensionFilters` parameter to filter the usage data returned
          for export by adding specific values of non-numeric Dimension data fields on
          included Meters. Only the data collected for the values you've added for the
          selected Dimension fields will be included in the export.
        - You can use the `aggregations` to apply aggregation methods the usage data
          returned for export. This restricts the range of usage data returned for
          export to only the data collected by aggregated fields on selected Meters.
          Nothing is returned for any non-aggregated fields on Meters. The usage data
          for Meter fields is returned as the values resulting from applying the
          selected aggregation method. See the
          [Aggregations for Queries - Options and Consequences](https://www.m3ter.com/docs/guides/data-explorer/usage-data-explorer-v2#aggregations-for-queries---understanding-options-and-consequences)
          for more details.
        - If you've applied `aggregations` to the usage returned for export, you can
          then use the `groups` parameter to group the data by _Account_, _Dimension_,
          or _Time_.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the ad-hoc export by a date/time range, use the `startDate`
        date/time request parameter to specify the start of the time period. The export
        will include all operational data from the specified `startDate` up until the
        date/time the export job runs.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the ad-hoc export by date/time range, use the `startDate` and `endDate`
        date/time parameters:

        - Both `startDate` and `endDate` are required.
        - `endDate` must be after `startDate`.
        - `endDate` cannot be after tomorrow at midnight UTC. For example if today is
          May 20th 2025, you can only choose `endDate` to be equal or before
          2025-05-21T00:00:00.000Z.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          operational_data_types: The list of the operational data types should be exported for.

          source_type: The type of data to export. Possible values are: OPERATIONAL

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        source_type: Literal["USAGE"],
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestAggregation] | Omit = omit,
        dimension_filters: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestDimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          ad-hoc export.
        - You can use the `dimensionFilters` parameter to filter the usage data returned
          for export by adding specific values of non-numeric Dimension data fields on
          included Meters. Only the data collected for the values you've added for the
          selected Dimension fields will be included in the export.
        - You can use the `aggregations` to apply aggregation methods the usage data
          returned for export. This restricts the range of usage data returned for
          export to only the data collected by aggregated fields on selected Meters.
          Nothing is returned for any non-aggregated fields on Meters. The usage data
          for Meter fields is returned as the values resulting from applying the
          selected aggregation method. See the
          [Aggregations for Queries - Options and Consequences](https://www.m3ter.com/docs/guides/data-explorer/usage-data-explorer-v2#aggregations-for-queries---understanding-options-and-consequences)
          for more details.
        - If you've applied `aggregations` to the usage returned for export, you can
          then use the `groups` parameter to group the data by _Account_, _Dimension_,
          or _Time_.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the ad-hoc export by a date/time range, use the `startDate`
        date/time request parameter to specify the start of the time period. The export
        will include all operational data from the specified `startDate` up until the
        date/time the export job runs.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the ad-hoc export by date/time range, use the `startDate` and `endDate`
        date/time parameters:

        - Both `startDate` and `endDate` are required.
        - `endDate` must be after `startDate`.
        - `endDate` cannot be after tomorrow at midnight UTC. For example if today is
          May 20th 2025, you can only choose `endDate` to be equal or before
          2025-05-21T00:00:00.000Z.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          source_type: The type of data to export. Possible values are: USAGE

          account_ids: List of account IDs for which the usage data will be exported.

          aggregations: List of aggregations to apply

          dimension_filters: List of dimension filters to apply

          end_date: The exclusive end date for the data export.

          groups: List of groups to apply

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["source_type"])
    def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
                "TRANSACTION_TYPES",
                "CHARGES",
            ]
        ]
        | Omit = omit,
        source_type: Literal["OPERATIONAL"] | Literal["USAGE"],
        version: int | Omit = omit,
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestAggregation] | Omit = omit,
        dimension_filters: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestDimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return self._post(
            f"/organizations/{org_id}/dataexports/adhoc",
            body=maybe_transform(
                {
                    "operational_data_types": operational_data_types,
                    "source_type": source_type,
                    "version": version,
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "end_date": end_date,
                    "groups": groups,
                    "meter_ids": meter_ids,
                },
                data_export_create_adhoc_params.DataExportCreateAdhocParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdHocResponse,
        )


class AsyncDataExportsResource(AsyncAPIResource):
    @cached_property
    def destinations(self) -> AsyncDestinationsResource:
        return AsyncDestinationsResource(self._client)

    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def schedules(self) -> AsyncSchedulesResource:
        return AsyncSchedulesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDataExportsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDataExportsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDataExportsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/m3ter-com/m3ter-sdk-python#with_streaming_response
        """
        return AsyncDataExportsResourceWithStreamingResponse(self)

    @overload
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
                "TRANSACTION_TYPES",
                "CHARGES",
            ]
        ],
        source_type: Literal["OPERATIONAL"],
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          ad-hoc export.
        - You can use the `dimensionFilters` parameter to filter the usage data returned
          for export by adding specific values of non-numeric Dimension data fields on
          included Meters. Only the data collected for the values you've added for the
          selected Dimension fields will be included in the export.
        - You can use the `aggregations` to apply aggregation methods the usage data
          returned for export. This restricts the range of usage data returned for
          export to only the data collected by aggregated fields on selected Meters.
          Nothing is returned for any non-aggregated fields on Meters. The usage data
          for Meter fields is returned as the values resulting from applying the
          selected aggregation method. See the
          [Aggregations for Queries - Options and Consequences](https://www.m3ter.com/docs/guides/data-explorer/usage-data-explorer-v2#aggregations-for-queries---understanding-options-and-consequences)
          for more details.
        - If you've applied `aggregations` to the usage returned for export, you can
          then use the `groups` parameter to group the data by _Account_, _Dimension_,
          or _Time_.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the ad-hoc export by a date/time range, use the `startDate`
        date/time request parameter to specify the start of the time period. The export
        will include all operational data from the specified `startDate` up until the
        date/time the export job runs.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the ad-hoc export by date/time range, use the `startDate` and `endDate`
        date/time parameters:

        - Both `startDate` and `endDate` are required.
        - `endDate` must be after `startDate`.
        - `endDate` cannot be after tomorrow at midnight UTC. For example if today is
          May 20th 2025, you can only choose `endDate` to be equal or before
          2025-05-21T00:00:00.000Z.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          operational_data_types: The list of the operational data types should be exported for.

          source_type: The type of data to export. Possible values are: OPERATIONAL

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @overload
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        source_type: Literal["USAGE"],
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestAggregation] | Omit = omit,
        dimension_filters: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestDimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        version: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        """Trigger an ad-hoc Data Export.

        Each ad-hoc Export can be configured for
        exporting _only one of_ either Usage or Operational data:

        **Operational Data Exports**.

        - **Entity Types**. Use the `operationalDataTypes` parameter to specify the
          entities whose operational data you want to include in the ad-hoc export.
        - **Export Files**. For each of the entity types you select, when the ad-hoc
          export runs a separate file is compiled containing the operational data for
          all entities of that type that exist in your Organization

        **Usage Data Exports**.

        - Select the Meters and Accounts whose usage data you want to include in the
          ad-hoc export.
        - You can use the `dimensionFilters` parameter to filter the usage data returned
          for export by adding specific values of non-numeric Dimension data fields on
          included Meters. Only the data collected for the values you've added for the
          selected Dimension fields will be included in the export.
        - You can use the `aggregations` to apply aggregation methods the usage data
          returned for export. This restricts the range of usage data returned for
          export to only the data collected by aggregated fields on selected Meters.
          Nothing is returned for any non-aggregated fields on Meters. The usage data
          for Meter fields is returned as the values resulting from applying the
          selected aggregation method. See the
          [Aggregations for Queries - Options and Consequences](https://www.m3ter.com/docs/guides/data-explorer/usage-data-explorer-v2#aggregations-for-queries---understanding-options-and-consequences)
          for more details.
        - If you've applied `aggregations` to the usage returned for export, you can
          then use the `groups` parameter to group the data by _Account_, _Dimension_,
          or _Time_.

        **Date Range for Operational Data Exports**. To restrict the operational data
        included in the ad-hoc export by a date/time range, use the `startDate`
        date/time request parameter to specify the start of the time period. The export
        will include all operational data from the specified `startDate` up until the
        date/time the export job runs.

        **Date Range for Usage Data Exports**. To restrict the usage data included in
        the ad-hoc export by date/time range, use the `startDate` and `endDate`
        date/time parameters:

        - Both `startDate` and `endDate` are required.
        - `endDate` must be after `startDate`.
        - `endDate` cannot be after tomorrow at midnight UTC. For example if today is
          May 20th 2025, you can only choose `endDate` to be equal or before
          2025-05-21T00:00:00.000Z.

        **NOTE:** You can use the ExportJob `id` returned to check the status of the
        triggered ad-hoc export. See the
        [ExportJob](https://www.m3ter.com/docs/api#tag/ExportJob) section of this API
        Reference.

        Args:
          source_type: The type of data to export. Possible values are: USAGE

          account_ids: List of account IDs for which the usage data will be exported.

          aggregations: List of aggregations to apply

          dimension_filters: List of dimension filters to apply

          end_date: The exclusive end date for the data export.

          groups: List of groups to apply

          meter_ids: List of meter IDs for which the usage data will be exported.

          version:
              The version number of the entity:

              - **Create entity:** Not valid for initial insertion of new entity - _do not use
                for Create_. On initial Create, version is set at 1 and listed in the
                response.
              - **Update Entity:** On Update, version is required and must match the existing
                version because a check is performed to ensure sequential versioning is
                preserved. Version is incremented by 1 and listed in the response.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        ...

    @required_args(["operational_data_types", "source_type"], ["source_type"])
    async def create_adhoc(
        self,
        *,
        org_id: str | None = None,
        operational_data_types: List[
            Literal[
                "BILLS",
                "COMMITMENTS",
                "ACCOUNTS",
                "BALANCES",
                "CONTRACTS",
                "ACCOUNT_PLANS",
                "AGGREGATIONS",
                "PLANS",
                "PRICING",
                "PRICING_BANDS",
                "BILL_LINE_ITEMS",
                "METERS",
                "PRODUCTS",
                "COMPOUND_AGGREGATIONS",
                "PLAN_GROUPS",
                "PLAN_GROUP_LINKS",
                "PLAN_TEMPLATES",
                "BALANCE_TRANSACTIONS",
                "TRANSACTION_TYPES",
                "CHARGES",
            ]
        ]
        | Omit = omit,
        source_type: Literal["OPERATIONAL"] | Literal["USAGE"],
        version: int | Omit = omit,
        account_ids: SequenceNotStr[str] | Omit = omit,
        aggregations: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestAggregation] | Omit = omit,
        dimension_filters: Iterable[data_export_create_adhoc_params.AdHocUsageDataRequestDimensionFilter] | Omit = omit,
        end_date: Union[str, datetime] | Omit = omit,
        groups: Iterable[DataExplorerGroupParam] | Omit = omit,
        meter_ids: SequenceNotStr[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AdHocResponse:
        if org_id is None:
            org_id = self._client._get_org_id_path_param()
        if not org_id:
            raise ValueError(f"Expected a non-empty value for `org_id` but received {org_id!r}")
        return await self._post(
            f"/organizations/{org_id}/dataexports/adhoc",
            body=await async_maybe_transform(
                {
                    "operational_data_types": operational_data_types,
                    "source_type": source_type,
                    "version": version,
                    "account_ids": account_ids,
                    "aggregations": aggregations,
                    "dimension_filters": dimension_filters,
                    "end_date": end_date,
                    "groups": groups,
                    "meter_ids": meter_ids,
                },
                data_export_create_adhoc_params.DataExportCreateAdhocParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AdHocResponse,
        )


class DataExportsResourceWithRawResponse:
    def __init__(self, data_exports: DataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = to_raw_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> DestinationsResourceWithRawResponse:
        return DestinationsResourceWithRawResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> SchedulesResourceWithRawResponse:
        return SchedulesResourceWithRawResponse(self._data_exports.schedules)


class AsyncDataExportsResourceWithRawResponse:
    def __init__(self, data_exports: AsyncDataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = async_to_raw_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithRawResponse:
        return AsyncDestinationsResourceWithRawResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> AsyncSchedulesResourceWithRawResponse:
        return AsyncSchedulesResourceWithRawResponse(self._data_exports.schedules)


class DataExportsResourceWithStreamingResponse:
    def __init__(self, data_exports: DataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = to_streamed_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> DestinationsResourceWithStreamingResponse:
        return DestinationsResourceWithStreamingResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> SchedulesResourceWithStreamingResponse:
        return SchedulesResourceWithStreamingResponse(self._data_exports.schedules)


class AsyncDataExportsResourceWithStreamingResponse:
    def __init__(self, data_exports: AsyncDataExportsResource) -> None:
        self._data_exports = data_exports

        self.create_adhoc = async_to_streamed_response_wrapper(
            data_exports.create_adhoc,
        )

    @cached_property
    def destinations(self) -> AsyncDestinationsResourceWithStreamingResponse:
        return AsyncDestinationsResourceWithStreamingResponse(self._data_exports.destinations)

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._data_exports.jobs)

    @cached_property
    def schedules(self) -> AsyncSchedulesResourceWithStreamingResponse:
        return AsyncSchedulesResourceWithStreamingResponse(self._data_exports.schedules)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required

from .data_field_param import DataFieldParam

__all__ = ["DerivedFieldParam"]


class DerivedFieldParam(DataFieldParam, total=False):
    calculation: Required[str]
    """
    The calculation used to transform the value of submitted `dataFields` in usage
    data. Calculation can reference `dataFields`, `customFields`, or system
    `Timestamp` fields. _(Example: datafieldms datafieldgb)_
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Union, Optional
from datetime import date, datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["ChargeScheduleUpdateResponse"]


class ChargeScheduleUpdateResponse(BaseModel):
    id: str
    """The UUID of the entity."""

    charge_description: str = FieldInfo(alias="chargeDescription")
    """The description for Charges created by the Balance Charge Schedule.

    Used on Bills for Charge line items.
    """

    amount: Optional[float] = None
    """The amount of each Charge created by the Balance Charge Schedule."""

    balance_id: Optional[str] = FieldInfo(alias="balanceId", default=None)
    """
    The unique identifier (UUID) for the Balance this Balance Charge Schedule was
    created for.
    """

    bill_epoch: Optional[date] = FieldInfo(alias="billEpoch", default=None)
    """
    Specifies a billing cycle date (_in ISO-8601 format_) for when the first Bill is
    generated for Balance Charges created by the Schedule, and also acts as a
    reference for when in the Schedule period subsequent Bills are created for the
    defined `billFrequency`. If blank, then the relevant Epoch date from your
    Organization's configuration is used.
    """

    bill_frequency: Optional[Literal["DAILY", "WEEKLY", "MONTHLY", "ANNUALLY"]] = FieldInfo(
        alias="billFrequency", default=None
    )
    """Represents standard scheduling frequencies options for a job."""

    bill_frequency_interval: Optional[int] = FieldInfo(alias="billFrequencyInterval", default=None)
    """How often Bills are issued.

    For example, if billFrequency is `MONTHLY` and `billFrequencyInterval` is 3,
    Bills are issued every three months.
    """

    bill_in_advance: Optional[bool] = FieldInfo(alias="billInAdvance", default=None)
    """
    Specifies how Charges created by the Balance Charge Schedule are billed - either
    in arrears or in advance:

    - If `false` then billing is in arrears.
    - If `true` then billing is in advance.
    """

    code: Optional[str] = None
    """Unique short code for the Balance Charge Schedule."""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """
    The unique identifier (UUID) of the user who created the Balance Charge
    Schedule.
    """

    currency: Optional[str] = None
    """The currency of the Charges created by the Balance Charge Schedule."""

    custom_fields: Optional[Dict[str, Union[str, float]]] = FieldInfo(alias="customFields", default=None)
    """User defined fields enabling you to attach custom data.

    The value for a custom field can be either a string or a number.

    If `customFields` can also be defined for this entity at the Organizational
    level,`customField` values defined at individual level override values of
    `customFields` with the same name defined at Organization level.

    See
    [Working with Custom Fields](https://www.m3ter.com/docs/guides/creating-and-managing-products/working-with-custom-fields)
    in the m3ter documentation for more information.
    """

    dt_created: Optional[datetime] = FieldInfo(alias="dtCreated", default=None)
    """
    The date and time (_in ISO-8601 format_) when the Balance Charge Schedule was
    created.
    """

    dt_last_modified: Optional[datetime] = FieldInfo(alias="dtLastModified", default=None)
    """
    The date and time (_in ISO-8601 format_) when the Balance Charge Schedule was
    last modified.
    """

    last_modified_by: Optional[str] = FieldInfo(alias="lastModifiedBy", default=None)
    """
    The unique identifier (UUID) for the user who last modified the Balance Charge
    Schedule.
    """

    name: Optional[str] = None
    """The name of the Balance Charge Schedule."""

    next_run: Optional[datetime] = FieldInfo(alias="nextRun", default=None)
    """
    The date and time (_in ISO-8601 format_) when the next Charge will be created by
    the Balance Charge Schedule.
    """

    previous_run: Optional[datetime] = FieldInfo(alias="previousRun", default=None)
    """
    The date and time (_in ISO-8601 format_) when the previous Charge was generated
    by the Balance Charge Schedule.
    """

    service_period_end_date: Optional[datetime] = FieldInfo(alias="servicePeriodEndDate", default=None)
    """
    The service period end date (_in ISO-8601 format_) of the Balance Charge
    Schedule.
    """

    service_period_start_date: Optional[datetime] = FieldInfo(alias="servicePeriodStartDate", default=None)
    """
    The service period start date (_in ISO-8601 format_) of the Balance Charge
    Schedule.
    """

    unit_price: Optional[float] = FieldInfo(alias="unitPrice", default=None)
    """Unit price. If the Charge was created with `amount` only, then will be null."""

    units: Optional[float] = None
    """Number of units.

    If the Charge was created with `amount` only, then will be null.
    """

    version: Optional[int] = None
    """The version number:

    - **Create:** On initial Create to insert a new entity, the version is set at 1
      in the response.
    - **Update:** On successful Update, the version is incremented by 1 in the
      response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["LookupTableRevisionDataGenerateDownloadURLResponse"]


class LookupTableRevisionDataGenerateDownloadURLResponse(BaseModel):
    """Response containing the upload job URL details"""

    headers: Optional[Dict[str, str]] = None
    """The headers"""

    job_id: Optional[str] = FieldInfo(alias="jobId", default=None)
    """UUID of the upload job"""

    url: Optional[str] = None
    """The presigned URL"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["TransactionSummaryResponse"]


class TransactionSummaryResponse(BaseModel):
    balance_consumed: Optional[float] = FieldInfo(alias="balanceConsumed", default=None)
    """Amount consumed from the original balance"""

    expired_balance_amount: Optional[float] = FieldInfo(alias="expiredBalanceAmount", default=None)
    """Amount of the balance that expired without being used"""

    initial_credit_amount: Optional[float] = FieldInfo(alias="initialCreditAmount", default=None)

    rollover_consumed: Optional[float] = FieldInfo(alias="rolloverConsumed", default=None)
    """Amount consumed from rollover credit"""

    total_credit_amount: Optional[float] = FieldInfo(alias="totalCreditAmount", default=None)

    total_debit_amount: Optional[float] = FieldInfo(alias="totalDebitAmount", default=None)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from m3ter import M3ter, AsyncM3ter
from tests.utils import assert_matches_type
from m3ter.pagination import SyncCursor, AsyncCursor
from m3ter.types.data_exports import (
    ScheduleListResponse,
    ScheduleCreateResponse,
    ScheduleDeleteResponse,
    ScheduleUpdateResponse,
    ScheduleRetrieveResponse,
)

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSchedules:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @parametrize
    def test_method_create_overload_1(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_1(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_create_overload_1(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_1(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_create_overload_2(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.create(
            source_type="USAGE",
            time_period="TODAY",
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_method_create_with_all_params_overload_2(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.create(
            source_type="USAGE",
            time_period="TODAY",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_create_overload_2(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.create(
            source_type="USAGE",
            time_period="TODAY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_create_overload_2(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.create(
            source_type="USAGE",
            time_period="TODAY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_retrieve(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.retrieve(
            id="id",
        )
        assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_retrieve(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_retrieve(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_retrieve(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.data_exports.schedules.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    def test_method_update_overload_1(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_1(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_update_overload_1(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_1(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_1(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.data_exports.schedules.with_raw_response.update(
                id="",
                operational_data_types=["BILLS"],
                source_type="OPERATIONAL",
            )

    @parametrize
    def test_method_update_overload_2(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_method_update_with_all_params_overload_2(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_update_overload_2(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_update_overload_2(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_update_overload_2(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.data_exports.schedules.with_raw_response.update(
                id="",
                source_type="USAGE",
                time_period="TODAY",
            )

    @parametrize
    def test_method_list(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.list()
        assert_matches_type(SyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    def test_method_list_with_all_params(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.list(
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(SyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    def test_raw_response_list(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(SyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    def test_streaming_response_list(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(SyncCursor[ScheduleListResponse], schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_method_delete(self, client: M3ter) -> None:
        schedule = client.data_exports.schedules.delete(
            id="id",
        )
        assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

    @parametrize
    def test_raw_response_delete(self, client: M3ter) -> None:
        response = client.data_exports.schedules.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = response.parse()
        assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

    @parametrize
    def test_streaming_response_delete(self, client: M3ter) -> None:
        with client.data_exports.schedules.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = response.parse()
            assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    def test_path_params_delete(self, client: M3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.data_exports.schedules.with_raw_response.delete(
                id="",
            )


class TestAsyncSchedules:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @parametrize
    async def test_method_create_overload_1(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_1(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_1(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_1(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.create(
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_create_overload_2(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.create(
            source_type="USAGE",
            time_period="TODAY",
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_method_create_with_all_params_overload_2(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.create(
            source_type="USAGE",
            time_period="TODAY",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_create_overload_2(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.create(
            source_type="USAGE",
            time_period="TODAY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_create_overload_2(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.create(
            source_type="USAGE",
            time_period="TODAY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleCreateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_retrieve(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.retrieve(
            id="id",
        )
        assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.retrieve(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.retrieve(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleRetrieveResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.data_exports.schedules.with_raw_response.retrieve(
                id="",
            )

    @parametrize
    async def test_method_update_overload_1(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_1(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
            version=0,
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_1(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_1(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.update(
            id="id",
            operational_data_types=["BILLS"],
            source_type="OPERATIONAL",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_1(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.data_exports.schedules.with_raw_response.update(
                id="",
                operational_data_types=["BILLS"],
                source_type="OPERATIONAL",
            )

    @parametrize
    async def test_method_update_overload_2(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_method_update_with_all_params_overload_2(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
            account_ids=["string"],
            aggregations=[
                {
                    "field_code": "x",
                    "field_type": "DIMENSION",
                    "function": "SUM",
                    "meter_id": "x",
                }
            ],
            dimension_filters=[
                {
                    "field_code": "x",
                    "meter_id": "x",
                    "values": ["string"],
                }
            ],
            groups=[{"group_type": "ACCOUNT"}],
            meter_ids=["string"],
            version=0,
        )
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_update_overload_2(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_update_overload_2(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.update(
            id="id",
            source_type="USAGE",
            time_period="TODAY",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleUpdateResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_update_overload_2(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.data_exports.schedules.with_raw_response.update(
                id="",
                source_type="USAGE",
                time_period="TODAY",
            )

    @parametrize
    async def test_method_list(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.list()
        assert_matches_type(AsyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.list(
            ids=["string"],
            next_token="nextToken",
            page_size=1,
        )
        assert_matches_type(AsyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    async def test_raw_response_list(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(AsyncCursor[ScheduleListResponse], schedule, path=["response"])

    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(AsyncCursor[ScheduleListResponse], schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_method_delete(self, async_client: AsyncM3ter) -> None:
        schedule = await async_client.data_exports.schedules.delete(
            id="id",
        )
        assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncM3ter) -> None:
        response = await async_client.data_exports.schedules.with_raw_response.delete(
            id="id",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        schedule = await response.parse()
        assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncM3ter) -> None:
        async with async_client.data_exports.schedules.with_streaming_response.delete(
            id="id",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            schedule = await response.parse()
            assert_matches_type(ScheduleDeleteResponse, schedule, path=["response"])

        assert cast(Any, response.is_closed) is True

    @parametrize
    async def test_path_params_delete(self, async_client: AsyncM3ter) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.data_exports.schedules.with_raw_response.delete(
                id="",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["LookupTableRevisionDataUpdateParams"]


class LookupTableRevisionDataUpdateParams(TypedDict, total=False):
    org_id: Annotated[str, PropertyInfo(alias="orgId")]

    lookup_table_id: Required[Annotated[str, PropertyInfo(alias="lookupTableId")]]

    items: Required[Iterable[Dict[str, object]]]
    """The data for a lookup table revision"""

    additional: SequenceNotStr[str]
    """Comma separated list of additional fields.

    For example, you can use `additional=lookupKey` to get the lookup key returned
    for each Data item. You can then use a lookup key for the Get/Upsert/Delete data
    entry endpoints in this section.
    """

    version: int
    """The version of the LookupTableRevisionData."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["LookupTableRevisionDataCopyResponse"]


class LookupTableRevisionDataCopyResponse(BaseModel):
    job_id: Optional[str] = FieldInfo(alias="jobId", default=None)
    """UUID of the Revision Data copy job."""

    revision_id: Optional[str] = FieldInfo(alias="revisionId", default=None)
    """The ID of the destination Revision."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .lookup_table_revision_response import LookupTableRevisionResponse as LookupTableRevisionResponse
from .lookup_table_revision_list_params import LookupTableRevisionListParams as LookupTableRevisionListParams
from .lookup_table_revision_create_params import LookupTableRevisionCreateParams as LookupTableRevisionCreateParams
from .lookup_table_revision_update_params import LookupTableRevisionUpdateParams as LookupTableRevisionUpdateParams
from .lookup_table_revision_data_copy_params import (
    LookupTableRevisionDataCopyParams as LookupTableRevisionDataCopyParams,
)
from .lookup_table_revision_data_copy_response import (
    LookupTableRevisionDataCopyResponse as LookupTableRevisionDataCopyResponse,
)
from .lookup_table_revision_data_update_params import (
    LookupTableRevisionDataUpdateParams as LookupTableRevisionDataUpdateParams,
)
from .lookup_table_revision_data_archieve_params import (
    LookupTableRevisionDataArchieveParams as LookupTableRevisionDataArchieveParams,
)
from .lookup_table_revision_data_delete_response import (
    LookupTableRevisionDataDeleteResponse as LookupTableRevisionDataDeleteResponse,
)
from .lookup_table_revision_data_retrieve_params import (
    LookupTableRevisionDataRetrieveParams as LookupTableRevisionDataRetrieveParams,
)
from .lookup_table_revision_data_update_response import (
    LookupTableRevisionDataUpdateResponse as LookupTableRevisionDataUpdateResponse,
)
from .lookup_table_revision_update_status_params import (
    LookupTableRevisionUpdateStatusParams as LookupTableRevisionUpdateStatusParams,
)
from .lookup_table_revision_data_archieve_response import (
    LookupTableRevisionDataArchieveResponse as LookupTableRevisionDataArchieveResponse,
)
from .lookup_table_revision_data_delete_key_params import (
    LookupTableRevisionDataDeleteKeyParams as LookupTableRevisionDataDeleteKeyParams,
)
from .lookup_table_revision_data_retrieve_response import (
    LookupTableRevisionDataRetrieveResponse as LookupTableRevisionDataRetrieveResponse,
)
from .lookup_table_revision_data_update_key_params import (
    LookupTableRevisionDataUpdateKeyParams as LookupTableRevisionDataUpdateKeyParams,
)
from .lookup_table_revision_data_delete_key_response import (
    LookupTableRevisionDataDeleteKeyResponse as LookupTableRevisionDataDeleteKeyResponse,
)
from .lookup_table_revision_data_update_key_response import (
    LookupTableRevisionDataUpdateKeyResponse as LookupTableRevisionDataUpdateKeyResponse,
)
from .lookup_table_revision_data_retrieve_key_response import (
    LookupTableRevisionDataRetrieveKeyResponse as LookupTableRevisionDataRetrieveKeyResponse,
)
from .lookup_table_revision_data_generate_download_url_params import (
    LookupTableRevisionDataGenerateDownloadURLParams as LookupTableRevisionDataGenerateDownloadURLParams,
)
from .lookup_table_revision_data_generate_download_url_response import (
    LookupTableRevisionDataGenerateDownloadURLResponse as LookupTableRevisionDataGenerateDownloadURLResponse,
)
